from PIL import Image

from LevelLoader import LevelLoader
from gui import GUIFacadeFactory, KeyEvent, Keyboard

# Load level
levelLoader = LevelLoader("level.tmx")
levelWidth = levelLoader.width
levelHeight = levelLoader.height
groundTileCoords, groundTexture, groundTileWidth, groundTileHeight = levelLoader.decodeLayer(0)
waterTileCoords, waterTexture, waterTileWidth, waterTileHeight = levelLoader.decodeLayer(1)
treeTileCoords, treeTexture, treeTileWidth, treeTileHeight = levelLoader.decodeLayer(2)

# Create window
guiFacade = GUIFacadeFactory().createInstance("OpenGL")
guiFacade.createWindow(
    "OpenGL 2D Facade - https://www.patternsgameprog.com/",
    1280, 768
)

# Ground layer
groundLayer = guiFacade.createLayer()
groundLayer.setTexture(groundTexture, groundTileWidth, groundTileHeight)
groundLayer.setGridMesh(groundTileCoords)

# Water layer
waterLayer = guiFacade.createLayer()
waterLayer.setTexture(waterTexture, waterTileWidth, waterTileHeight)
waterLayer.setGridMesh(waterTileCoords)

# Tree layer
treeLayer = guiFacade.createLayer()
treeLayer.setTexture(treeTexture, treeTileWidth, treeTileHeight)
treeLayer.setGridMesh(treeTileCoords)

# Main loop
guiFacade.init()
translationX = 0.0
translationY = 0.0
minTranslationX = 0
minTranslationY = 0
maxTranslationX = levelWidth - guiFacade.screenWidth / groundTileWidth
maxTranslationY = levelHeight - guiFacade.screenHeight / groundTileHeight
while not guiFacade.closingRequested:
    # Update inputs state
    guiFacade.updateInputs()

    # Keyboard key single press
    keyboard = guiFacade.keyboard
    for keyEvent in keyboard.keyEvents:
        if keyEvent.type == KeyEvent.KEYDOWN:
            if keyEvent.key == Keyboard.K_ESCAPE:
                guiFacade.closingRequested = True
                break
    keyboard.clearKeyEvents()

    # Keyboard key multi/continuous press
    if keyboard.isKeyPressed(Keyboard.K_LEFT):
        translationX -= 1.0
    if keyboard.isKeyPressed(Keyboard.K_RIGHT):
        translationX += 1.0
    if keyboard.isKeyPressed(Keyboard.K_UP):
        translationY -= 1.0
    if keyboard.isKeyPressed(Keyboard.K_DOWN):
        translationY += 1.0

    if translationX < minTranslationX:
        translationX = minTranslationX
    if translationY < minTranslationY:
        translationY = minTranslationY
    if translationX >= maxTranslationX:
        translationX = maxTranslationX
    if translationY >= maxTranslationY:
        translationY = maxTranslationY

    guiFacade.setTranslation(translationX, translationY)

    # Render scene
    guiFacade.render()

# Screen capture (to check that the rendering is ok)
capture = guiFacade.captureScreenContent(0, 0, guiFacade.screenWidth, guiFacade.screenHeight)
Image.fromarray(capture).save("capture.png")

# Delete all objects and quit
guiFacade.quit()
